#!/bin/bash
    clear
    
    catch_exits() {  # function to clean up install files if script is aborted
        echo "Removing reef-pi Installation Files             ...takes less than 1 minute"
        CWD="$( cd -P "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
        sudo rm -r "${CWD}" &> /dev/null
        sudo rm -r "reef-pi_install_files.zip" &> /dev/null
        sudo rm -r reef-pi-5.3-pi*.deb &> /dev/null
        exit 0
        printf "\n$(basename $0): exiting\n" 1>&2
        rm -rf $DIR
        exit 1
    }

    trap catch_exits 1 2 3 15 20  # run clean up if script is aborted

    cyan='\033[0;36m'
    NC='\033[0m' # No Color
    bold=$(tput bold)
    normal=$(tput sgr0)
    
    echo ""
    echo ""
    echo ""
    echo "This script will automate the installation process for reef-pi but does require attending."
    echo ""
    echo ""
    echo "This script does the following"
    echo "    * updates Raspberry Pi Operating System"
    echo "    * updates Raspberry Pi password"
    echo "    * prompts for new Raspbbery Pi user account"
    echo "    * prompts for hostname"
    echo "    * prompts for time zone"
    echo "    * enables time sync"
    echo "    * installs reef-pi"
    echo "    * configures reef-pi"
    echo "    * generates website address to access reef-pi"
    echo "    * removes reef-pi installation files"
    echo "    * expands file system on microSD card"
    echo ""
    echo ""
    echo ""
    echo "Press enter to continue or press ${bold}Ctrl + C${normal} to abort"
    read input
    echo ""
    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"


    # check if sudo is used
    if [ "$(id -u)" != 0 ]; then
        echo ""
        echo ""
        echo "You need to run this script with ${bold}sudo${normal} at the beginning of the command"
        echo ""
        echo ""
        echo ""
        echo ""
        exit 1
    fi


    # update password
    echo ""
    echo ""
    echo "Type in a new password for the Raspberry Pi and press ${bold}ENTER${normal} on your keyboard" 
    echo "(note the cursor won't move as you type)"
    sudo passwd pi
    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"


    # create new user
    echo ""
    echo ""
    echo "Do you want to create a new user account?"
    echo "Type Y or N and press ${bold}ENTER${normal} on your keyboard"
    echo ""
    echo "    ${bold}Y${normal} = Create new user account"
    echo "    ${bold}N${normal} = No thanks"
    echo ""
    read addusr
    valid=0
    while (( $valid <= 0 ))  # continue to ask for for board type until valid one is entered
    do
        if [ -z "$addusr" ]  # check if board type is empty
        then
            echo "Selection not valid, please try again"
            read addusr
        else
            if [ $addusr = y ] || [ $addusr = Y ] || [ $addusr = Yes ] || [ $addusr = yes ] || [ $addusr = YES ]
            then
                echo ""
                echo -e "${cyan}________________________________________________________________________________________${NC}"
                echo ""
                echo ""
                echo "Type in your a new username and password for the Raspberry Pi and press ${bold}ENTER${normal}"
                echo "on your keyboard"
                read newusr
                sudo useradd -m $newusr &> /dev/null
                # add new user to same groups as pi user
                sudo usermod $newusr -a -G pi,adm,dialout,cdrom,sudo,audio,video,plugdev,games,users,input,netdev,spi,i2c,gpio &> /dev/null
                # set passwd for new user
                sudo passwd $newusr
                echo ""
                echo "New user was created"
                echo ""
                valid=1
            elif [ $addusr = n ] || [ $addusr = N ] || [ $addusr = No ] || [ $addusr = no ] || [ $addusr = NO ]
            then
                echo ""
                valid=1
            else
                echo "Selection not valid, please try again"
                read addusr
                valid=0
            fi
        fi
    done
    


    # update hostname
    echo -e "${cyan}________________________________________________________________________________________${NC}"
    echo ""
    echo ""
    echo "Type in a hostname for reef-pi and press ${bold}ENTER${normal} on your keyboard, this is similar to"
    echo "a domain name to access a website."
    read new_hostname

    while [ -z "$new_hostname" ]  # continue to ask for hostname until one in entered
    do
        if [ -z "$new_hostname" ]  # check if hostname is empty
        then
            echo "Hostname was blank, please try again."
            read new_hostname
        fi
    done

    sudo raspi-config nonint do_hostname $new_hostname &> /dev/null
    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"


    # update timezone
    echo ""
    echo ""
    echo "Press ${bold}ENTER${normal} on your keyboard to select your Timezone from the list"
    read input
    dpkg-reconfigure tzdata
    echo -e "${cyan}________________________________________________________________________________________${NC}"


    # install reef-pi
    echo ""
    echo ""
    echo "Type in a number from the list below for the Raspberry Pi you will use and press"
    echo "${bold}ENTER${normal} on your keyboard"
    echo ""
    echo "    ${bold}1${normal} = Raspberry Pi Zero W or Zero 2 W"
    echo "    ${bold}2${normal} = Raspberry Pi 3"
    echo "    ${bold}3${normal} = Raspberry Pi 4"
    echo ""
    valid=0
    read pi

    while (( $valid <= 0 ))  # continue to ask for Pi version until valid one is entered
    do
        if [ -z "$pi" ]  # check if Pi version is empty
        then
            echo "Selection not valid, please try again"
            read pi
        else
            if [ $pi = 1 ]
            then
                rasp="Raspberry Pi Zero W"
                valid=1
            elif [ $pi = 2 ]
            then
                rasp="Raspberry Pi 3     "
                valid=1
            elif [ $pi = 3 ]
            then
                rasp="Raspberry Pi 4     "
                valid=1
            else
                echo "Selection not valid, please try again"
                read pi
                valid=0
            fi
        fi
    done

    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"
    echo ""
    echo ""
    echo "Would you like to setup reef-pi for a Robo-Tank Controller?"
    echo "Type Y or N and press ${bold}ENTER${normal} on your keyboard"
    echo ""
    echo "    ${bold}Y${normal} = yes setup for Robo-Tank"
    echo "    ${bold}N${normal} = no I will setup manually"
    echo ""
    read board

    robotank=0
    valid=0
    while (( $valid <= 0 ))  # continue to ask for for board type until valid one is entered
    do
        if [ -z "$board" ]  # check if board type is empty
        then
            echo "Selection not valid, please try again"
            read board
        else
            if [ $board = y ] || [ $board = Y ] || [ $board = Yes ] || [ $board = yes ] || [ $board = YES ]
            then
                robotank=1
                echo ""
                echo -e "${cyan}________________________________________________________________________________________${NC}"
                echo ""
                echo ""
                echo "Which Robo-Tank Controller do you have? Type in a number from the list below and" 
                echo "and press ${bold}ENTER${normal} on your keyboard"
                echo ""
                echo "    ${bold}1${normal} = Robo-Tank Version 1"
                echo "    ${bold}2${normal} = Robo-Tank Version 2"
                echo "    ${bold}3${normal} = Robo-Tank Version 3 - Latest"
                echo ""
                read boardV

                while (( $valid <= 0 ))  # continue to ask for board version until valid one is entered
                do
                    if [ -z "$boardV" ]  # check if board version is empty
                    then
                        echo "Selection not valid, please try again"
                        read boardV
                    else
                        if [ $boardV = 1 ]
                        then
                            valid=1
                        elif [ $boardV = 2 ]
                        then
                            valid=1
                        elif [ $boardV = 3 ]
                        then
                            valid=1
                        else
                            echo "Selection not valid, please try again"
                            read boardV
                            valid=0
                        fi
                    fi
                done

                valid=1
            elif [ $board = n ] || [ $board = N ] || [ $board = No ] || [ $board = no ] || [ $board = NO ]
            then
                robotank=0
                valid=1
            else
                echo "Selection not valid, please try again"
                read board
                valid=0
            fi
        fi
    done

    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"
    echo ""
    echo ""
    echo -e "${cyan}The installtion will now begin, once it finishes you'll be prompted.${NC}"
    echo ""
    echo ""
    echo ""


    # # sync time
    echo "Syncing time                                    ...takes less than 1 minute"
    sudo systemctl start systemd-timesyncd &> /dev/null
    sudo systemctl enable systemd-timesyncd &> /dev/null


    # update Raspbian
    apt update &> /dev/null && apt upgrade -y &> /dev/null && apt autoremove -y &> /dev/null && apt install -y i2c-tools &> /dev/null & PID=$!
    echo "Updating Raspberry Pi Operating System          ...takes 1-30 minutes"
    # While process is running...
    while kill -0 $PID 2> /dev/null; do
    printf "\e[1;33m▓"
    sleep 15
    done
    echo ""
    printf "\e[0;37m"

    # Install reef-pi
    echo "${normal}Installing reef-pi on a" $rasp "         ...takes about 1 minute"

    if [ $pi = 1 ]
    then
        wget -c https://github.com/reef-pi/reef-pi/releases/download/5.3/reef-pi-5.3-pi0.deb &> /dev/null
        sudo dpkg -i reef-pi-5.3-pi0.deb &> /dev/null
    elif [ $pi = 2 ]
    then
        wget -c https://github.com/reef-pi/reef-pi/releases/download/5.3/reef-pi-5.3-pi3.deb &> /dev/null
        sudo dpkg -i reef-pi-5.3-pi3.deb &> /dev/null
    else
        wget -c https://github.com/reef-pi/reef-pi/releases/download/5.3/reef-pi-5.3-pi3.deb &> /dev/null
        sudo dpkg -i reef-pi-5.3-pi3.deb &> /dev/null
    fi


    echo "Backing up Raspberry Pi /boot/config.txt file   ...takes about 1 minute"
    # Verify if config.org exists and if not create a backup config.txt file for historical purposes
    FILE=/boot/config.org
    if [ ! -f "$FILE" ]; then
        sudo cp /boot/config.txt /boot/config.org &> /dev/null
    fi

    if grep -q 'i2c-bcm2708' /etc/modules; then
        echo ""
    else
        echo 'i2c-bcm2708' >> /etc/modules
    fi
    if grep -q 'i2c-dev' /etc/modules; then
        echo ""
    else
        echo 'i2c-dev' >> /etc/modules
    fi
    
    # Enable 1wire, rpi-pwm and i2c and set baudrate
    sudo sed -i 's/#dtparam=i2c_arm=on$/dtparam=i2c_arm=on\ndtparam=i2c_arm_baudrate=10000\ndtoverlay=pwm-2chan/g' /boot/config.txt &> /dev/null
    sudo raspi-config nonint do_onewire 0 &> /dev/null
    # Disable spi
    sudo raspi-config nonint do_spi 1 &> /dev/null
    
    sudo systemctl start reef-pi.service &> /dev/null
    sudo systemctl enable reef-pi.service &> /dev/null

    if [ $robotank = 1 ]
    then
        echo "Installing database for Robo-Tank               ...takes about 1 minute"
        sudo systemctl stop reef-pi.service &> /dev/null

        if [ $boardV = 1 ]
        then
            CWD="$( cd -P "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
            sudo cp "${CWD}/v1/reef-pi.db" /var/lib/reef-pi/reef-pi.db &> /dev/null
        elif [ $boardV = 2 ]
        then
            CWD="$( cd -P "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
            sudo cp "${CWD}/v2/reef-pi.db" /var/lib/reef-pi/reef-pi.db &> /dev/null
            sudo sed -i 's/dtoverlay=w1-gpio$/dtoverlay=w1-gpio,gpiopin=4\ndtoverlay=w1-gpio,gpiopin=23/g' /boot/config.txt &> /dev/null
        else
            CWD="$( cd -P "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
            sudo cp "${CWD}/v3/reef-pi.db" /var/lib/reef-pi/reef-pi.db &> /dev/null
            sudo sed -i 's/dtoverlay=w1-gpio$/dtoverlay=w1-gpio,gpiopin=4\ndtoverlay=w1-gpio,gpiopin=22/g' /boot/config.txt &> /dev/null
        fi

        sudo systemctl start reef-pi.service &> /dev/null
        sleep 3
    else
        echo "Updating /boot/config.txt                       ...takes about 1 minute"
        sudo systemctl stop reef-pi.service &> /dev/null
        sudo rm -rf /var/lib/reef-pi/reef-pi.db &> /dev/null
        sudo systemctl start reef-pi.service &> /dev/null
        sleep 3
    fi


    echo "Removing reef-pi Installation Files             ...takes less than 1 minute"
    CWD="$( cd -P "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
    sudo rm -r "${CWD}" &> /dev/null
    sudo rm -r "reef-pi_install_files.zip" &> /dev/null
    sudo rm -r reef-pi-5.3-pi*.deb &> /dev/null
    echo ""
    echo -e "${cyan}________________________________________________________________________________________${NC}"
    echo ""
    echo ""
    echo ""
    echo ""
    echo ""
    echo "You have successfully installed reef-pi, you can now press ${bold}ENTER${normal} on your keyboard to"
    echo "restart your Raspberry Pi."
    echo ""
    echo "After the Raspberry Pi restarts open a web browser from any device on your local "
    echo "network and type one of the links below. The first link is the IP address of the"
    echo "Raspberry Pi, the 2nd link is the hostname you added during the installation. Write"
    echo "these down or create a bookmark in your browser. Note it is possible your network"
    echo "changes the IP address for the Raspberry Pi and the link no longer works, this is"
    echo "why we setup a hostname as this will never change."
    echo ""
    ip= hostname -I | awk '{ print "http://" $1 }'
    echo "${bold}http://$new_hostname${normal}"
    echo ""
    echo ""

    echo "The default login for reef-pi is as follows."
    echo ""
    echo "Username = ${bold}reef-pi${normal}"
    echo "Password = ${bold}reef-pi${normal}"
    echo ""
    echo ""
    echo "Have a great day!"
    echo ""
    echo ""
    echo ""
    echo "Press ${bold}ENTER${normal} on your keyboard to restart the Raspberry Pi."
    echo ""
    echo ""
    echo ""
    
    read input
    # # # expand file system
    sudo raspi-config --expand-rootfs &> /dev/null
    sudo reboot